// Driver File for ADE7753
// this driver uses PIC's hardware SPI, mode 1

#define CS PIN_C6
#define WRITE 0x80

float Vrms, Irms, Vgain, Igain, Period, Freq, P, Q, S, PF, Pgain, Qgain, Sgain;
signed int32 Vraw, Iraw, Praw, Qraw, Sraw;
int32 Ioff;

int32 Read24(int Adress)
{
   int i;
   int32 result = 0;
   output_low(CS);
   spi_write(Adress);
   for (i=0;i<3;i++)
   {
    delay_us(5);
    result = result | ((int32)(spi_read(0))<<(8*(2-i)));
   }
   output_high(CS);
   return (result);
}

int16 Read16(int Adress)
{
   int i;
   int16 result = 0;
   output_low(CS);
   spi_write(Adress);
   for (i=0;i<2;i++)
   {
    delay_us(5);
    result = result | ((int16)(spi_read(0))<<(8*(1-i)));
   }
   output_high(CS);
   return (result);
}

int8 Read8(int Adress)
{
   int8 result;
   output_low(CS);
   spi_write(Adress);
   result = spi_read(0);
   output_high(CS);
   return (result);
}
            
void Write16(int Adress, int16 Data)
{
   int8 MSB, LSB;
   
   MSB = (int8)(Data>>8);
   LSB = (int8)(Data);
     
   output_low(CS);
   spi_write(Adress|WRITE);
   delay_us(5);
   spi_write(MSB);
   delay_us(5);
   spi_write(LSB);
   delay_us(5);
   output_high(CS);
}

void Write8(int Adress, int8 Data)
{
   output_low(CS);
   spi_write(Adress|WRITE);
   delay_us(5);
   spi_write(Data);
   delay_us(5);
   output_high(CS);
}

void ade_init(void)
{
    Write16(0x09, 0x0080); //Chip configuration
    Write16(0x1C, 120); //Line cycle accumulation for 120 half periods
}

int8 calcPGAValue(int gain)
{
   if (gain==1) return(0);
   if (gain==2) return(1);
   if (gain==4) return(2);
   if (gain==8) return(3);
   if (gain==16) return(4);
   return(0);
}

void ade_setPGA(int8 ch1gain, int8 ch2gain)
{
   int8 Out;
      
   ch1gain = calcPGAValue(ch1gain);
   ch2gain = calcPGAValue(ch2gain);
   
   Out = (ch2gain<<5)|(ch1gain);
   Write8(0x0F, Out);
}

void GetADCValues()
{
   //attente d'un passage  0 pour effectuer la mesure de Urms et Irms (ou timeout)
   while(!(Read16(0x0B)&0x1010)); 
     
   Vraw = (signed int32)(Read24(0x17) - 0x8000);
   Iraw = (signed int32)(Read24(0x16) - Ioff);
   Vrms = (float)(Vraw)*Vgain/1000.0;
   Irms = (float)(Iraw)*Igain/1000.0;
   Period = (float)(Read16(0x27))/448000;
   if (Vrms>10.0) Freq = 1.0/Period;
   else           Freq = 0.0;
}

void GetPowerLoop()
{
   int16 IRQStatus;
   
   IRQStatus = Read16(0x0C);
   if (IRQStatus&0x1000) //ZeroCross TimeOut
   {
      Praw = 0;
      Qraw = 0;
      Sraw = 0;
   }  
   else if (IRQStatus&0x0004) //Accumulation completed
   {
      Praw = (signed int32)(Read24(0x04)<<8);
      Qraw = (signed int32)(Read24(0x08)<<8);
      Sraw = (signed int32)(Read24(0x07)<<8);
   }
   if (Period>0.0)
   {
      P = (float)(Praw)*Pgain/(Period*10000.0);
      Q = (float)(Qraw)*Qgain/(Period*10000.0);
      S = (float)(Sraw)*Sgain/(Period*10000.0);
      if (S<1.0) { S = 0.0; PF = 0.0;}
      else PF = P/S;
     
   }
   else
   {
      P = 0.0;
      Q = 0.0;
      S = 0.0;
      PF = 0.0;
   }
}
